/**************************************************************************
EASY MSc Dissertation - Project
Ken Webb
August 4, 2004
BacteriumApp.h: interface for the BacteriumApp class.

This is the main application class used in the PBNN system.
It represents a bacterium or one entire system of proteins and other polymers.
**************************************************************************/

#if !defined(AFX_BACTERIUMAPP_H__727C76DA_3F52_4EFE_BE8F_857E26E530D8__INCLUDED_)
#define AFX_BACTERIUMAPP_H__727C76DA_3F52_4EFE_BE8F_857E26E530D8__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Application.h"
#include "Protein.h"
#include "SmallMolecule.h"
#include "LipidBilayer.h"
#include "Monomer.h"
#include "Probabilities.h"
#include "Statistics.h"
#include "Grid.h"
#include "Queue.h"
#include "Text.h"
#include "Msg.h"
#include "Rndm.h"
#include "GlutMaster.h"
#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Logic Types
const int NullLT = 0;
const int _0000_LT = 1;
const int _1000_LT = 2;
const int _0100_LT = 3;
const int _1100_LT = 4;
const int _0010_LT = 5;
const int _1010_LT = 6;
const int _0110_LT = 7;
const int _1110_LT = 8;
const int _0001_LT = 9;
const int _1001_LT = 10;
const int _0101_LT = 11;
const int _1101_LT = 12;
const int _0011_LT = 13;
const int _1011_LT = 14;
const int _0111_LT = 15;
const int _1111_LT = 16;
const int AndLT = _0001_LT;
const int OrLT  = _0111_LT;
const int XorLT = _0110_LT;

// Other constants
const int maxPolymers = 5000; // maximum number of polymers in the application
const int LigandsMultiplier = 5; // initially make numLigands * LigandsMultiplier ligands

// BacteriumApp
class BacteriumApp : public Application  
{
friend class ProteinFactory; // convenience routines to create proteins
public:
	BacteriumApp( GlutMaster* gm, Probabilities* pr, Statistics *st );
	virtual ~BacteriumApp();

	// inherited from Application superclass
	void CallBackDisplayFunc();
	void CallBackKeyboardFunc(unsigned char key, int x, int y);
	void CallBackMouseFunc(int button, int state, int x, int y);
	void CallBackReshapeFunc(int w, int h);   
	void CallBackTimerFunc(int value);
	void initialize( char *configFileName );

	void setQ( Queue *q, int direction ); // specify message queue(s)
	void glMain( int argc, char *argv[] ); // initialize GL

protected:
	// Read configuration info from a file
	void readConfigFromFile( char *fileName );
	void getTrialTypeFromFile( char* ttStr );
	void getLogicTypeFromFile( char* ltStr );

	// Called by OpenGL timer function (initialization actions)
	void initSim_1( int tVal ); // Logic Types application
	void initSim_2( int tVal ); // test
	void initSim_3( int tVal ); // Bacterial Chemotaxis
	void initSim_4( int tVal ); // test
	void initSim_5( int tVal ); // Bacterial Chemotaxis - no initial positioning
	void initSim_6( int tVal ); // Bacterial Chemotaxis - larger bacterium
	void actSim( int tVal ); // Called by OpenGL timer function (regular actions)

	void timeExperiment( void); // perform certain actions at specific times
	void drawGrids(); // draw all grid info to OpenGL
	void drawGrid1( gint* monoId, float rOff, float gOff, float bOff ); // draw one grid
	void tRan( void ); // test random numbers
	void tGrid2shape( void ); // test grid2shape() function
	void addLigands( void ); // a convenience routine called by initSim functions

	Probabilities* prob; // ptr passed to all proteins, and to grid
	Statistics *stats; // ptr to the single statistics instance

	GlutMaster* glutMaster; // interface to OpenGL; allows multiple OpenGL windows

	// grid sizes, and OpenGL window size
	int gridSizeX;
	int gridSizeY;
	int winSizeX;
	int winSizeY;

	// Application number
	int appNum;

	// Logic Type
	int logicType;
	int trialType;

	// Number of screen coordinate "pixels" in a unit
	int unitSize; // default: 8   useful range: 1 to 10

	// Time step
	int timeStepCounter; // counter
	int itsPerTimeStep; // number of iterations per time step

	// Pause
	bool pause;
	bool step;
	bool pauseDisplay; // continues timer loop, but doesn't display

	// Grid
	Grid *grid; // only one grid object for entire application

	// convenience variables used during configuration
	Protein *prot;
	Protein *tmpProt;
	SmallMolecule *lig;
	LipidBilayer *lipBilayer;

	// Configured quantities of various proteins
	int numCheY; // total number of CheY protein instances
	int numCheB; // total number of CheB protein instances
	int numCheR; // total number of CheR protein instances
	int numCheZ; // total number of CheZ protein instances

	// Configured quantities of various small molecules (SMs)
	int ligOffsetX; // offsets from default grid position
	int ligOffsetY;
	int numLigands; // number of ligands of each type
	int numAxp; // number of Atp and of Adp and of Amp
	int numMeth; // number of methanol SMs
	int numMeth0; // number of methanol base SMs (lack the actual methanol itself)

	// Monomer Types
	Monomers *mn;
	const MonomerType *mt;

	// flag to specify special actions during first call to a function
	bool firstTime;

	// Queues for communicating with EnvironmentApp
	Queue *outQ; // queue to send BactEnvPdu messages to server
	Queue *inQ;  // queue to receive BactEnvPdu messages from server
	// Receive a PDU message from EnvironmentApp, by checking the queue
	void rxPdu( void );
};

#endif // !defined(AFX_BACTERIUMAPP_H__727C76DA_3F52_4EFE_BE8F_857E26E530D8__INCLUDED_)
