/**************************************************************************
EASY MSc Dissertation - Project
Ken Webb
August 10, 2004
EnvironmentApp.cpp: implementation of the EnvironmentApp class.

This is an optional class, only used in client-server mode.
**************************************************************************/

#include "EnvironmentApp.h"
#include "BactEnvPdu.h"

// Experimental or Control Group
//#define CONTROL_GROUP 1 // comment out to use experimental group

// Graphics or No (minimal) graphics
//#define NO_GRAPHICS 1 // comment out to allow normal display of server graphics

// Increment ligands every n time steps
#define INCREMENT_LIGANDS 1

// Experiment Timing Constants
// These determine when certain things occur during different experiments.
// measured in time steps
#define TIMINGS_MODE 6

#if TIMINGS_MODE == 1
// Quick Test Mode
const int TimeToStartActing   = 1000; // start doing something
const int TimeToMopupLigands  = 4000; // remove ligands from environment
const int TimeToEndExperiment = 5000; // end experiment
const int TimeToCreateEaStats = 5000; // create statistics
const int TimeToStartTrial    = 999999999;

#elif TIMINGS_MODE == 2
// Default Mode
const int TimeToStartActing   =  50000; // start doing something
const int TimeToMopupLigands  = 180000; // remove ligands from environment
const int TimeToEndExperiment = 200000; // end experiment
const int TimeToCreateEaStats = 200000; // create stats
const int TimeToStartTrial    = 999999999;

#elif TIMINGS_MODE == 3
// Learn/Trial Mode
// set NumLigands: 400 ConcPatternDefault: 6
const int TimeToStartActing   =  50000; // stabilization finished; start the learning phase
const int TimeToStartTrial    = 100000; // learning finished; start the trial phase
const int TimeToEndExperiment = 200000; // trial phase finished; end the experiment
const int TimeToCreateEaStats = 200000; // create stats
const int TimeToMopupLigands  = 999999999;

#elif TIMINGS_MODE == 4
// No Ligands Mode
const int TimeToStartActing   =  50000; // start doing something
// Although there are no ligands in this mode, still "remove" them so do everything
// is done the same way as in the default mode.
const int TimeToMopupLigands  = 180000; // remove ligands from environment
const int TimeToEndExperiment = 200000; // end experiment
const int TimeToCreateEaStats = 999999999; // these stats are meaningless when no ligand
const int TimeToStartTrial    = 999999999;

#elif TIMINGS_MODE == 5
// Impulse Mode
// set NumLigands: 400 ConcPatternDefault: 5
const int TimeToStartActing   =  50000; // start doing something
const int TimeToMopupLigands  =  60000; // remove ligands from environment
const int TimeToEndExperiment =  80000; // end experiment
const int TimeToCreateEaStats = 999999999;
const int TimeToStartTrial    = 999999999;

#elif TIMINGS_MODE == 6
// Impulse Mode
// set NumLigands: 3 ConcPatternDefault: 5
const int TimeToStartActing   =  50000; // start doing something
const int TimeToMopupLigands  =  80000; // remove ligands from environment
const int TimeToEndExperiment =  100000; // end experiment
const int TimeToCreateEaStats = 999999999;
const int TimeToStartTrial    = 999999999;

#endif

// Other Constants
const int ligandStartRow = 150;
const int ligandEndRow   = 550;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

EnvironmentApp::EnvironmentApp( GlutMaster* gm, Statistics* st )
{
	glutMaster = gm;
	stats = st;
	gridSizeX = 120;
	gridSizeY = 120;
	unitSize = 1; // default: 1
	timeStepCounter = 0;
	itsPerTimeStep = 1;
	step = false;
	pause = false;
	firstTime = true;
	penStatus = PenDown;

	// Bacteria and their associated queues
	maxBacteria = 2; // usually only use 1, and sometimes 2
	bacterium = new BacteriumProxy*[maxBacteria];
	for (int i = 0; i < maxBacteria; i++)
		bacterium[i] = new BacteriumProxy();
	numCurrentBacteria = 0;
	outQ = new Queue*[maxBacteria]; // allocate space for array of ptrs to queues
	inQ = new Queue*[maxBacteria];

	NumLigands = 100; // true value will be read in from file
	ligandConcBefore = 0;
	ligandConcAfter = 0;
	ConcPatternDefault = 5; // true value will be read in from file
	bGrid = NULL;
	eGrid = NULL;
	eGridChanged = true; // make sure eGrid is displayed first time
}

EnvironmentApp::~EnvironmentApp()
{
}

//////////////////////////////////////////
// CallBackTimerFunc
// Called to initiate each time step.
// OpenGL (GlutMaster) callback function.
//////////////////////////////////////////
void EnvironmentApp::CallBackTimerFunc(int value)
{
	if (firstTime) {
		firstTime = false;
		initSim(value);
	}
	else
		actSim( value );
}

//////////////////////////////
// initSim
// Initialize the simulation.
//////////////////////////////
void EnvironmentApp::initSim( int tVal )
{
	// Initialize all bacteria proxies, each standing in for a bacterium
	// in its own client window.
	for (int i = 0; i < maxBacteria; i++) {
		if (i == 0) // first bacterium in center
			bacterium[i]->x = gridSizeX / 2;
		else // other bacterium offset from center
			bacterium[i]->x = gridSizeX / 2 - 100;
		if ((ConcPatternDefault == 4) || (ConcPatternDefault == 5) || (ConcPatternDefault == 6)) {
			if (NumLigands == 100)
				bacterium[i]->y = 350;
			else // NumLigands == 400
				bacterium[i]->y = 199; // exactly at half point
		}
		else
			bacterium[i]->y = ligandStartRow + 30;
		//printf( "|%d|\n", eGrid[bacterium[i]->y*gridSizeX] ); // confirm
		bacterium[i]->rotated = BactRot90; // face a neutral direction
		bacterium[i]->colorIntensity = 1.0f;
		// ligand counts are gathered for statistics purposes
		bacterium[i]->ligCount = new int[NumLigands];
		for (int j = 0; j < NumLigands; j++) // initialize counts to 0
			bacterium[i]->ligCount[j] = 0;
		// save initial x and y position for some experiments
		bacterium[i]->initialX = bacterium[i]->x;
		bacterium[i]->initialY = bacterium[i]->y;
		// store position for display
		bGrid[bacterium[i]->y * gridSizeX + bacterium[i]->x] = i;
	}
	glutPostRedisplay();
}

//////////////////////////////////////////////////////////////////
// actSim
// Regular action each time step. Process all items in receive Q.
// Do processing only if not paused, and if not single stepping.
//////////////////////////////////////////////////////////////////
void EnvironmentApp::actSim( int tVal )
{
	if (!pause && !globalPause) {
		if (timeStepCounter <= TimeToStartActing) {
			// empty all input queues and delete their contents; not processing these yet
			for (int bactIx = 0; bactIx < numCurrentBacteria; bactIx++)
				inQ[bactIx]->emptyDelete();
			// keep counting even though not yet processing
			timeStepCounter += itsPerTimeStep;
			if (timeStepCounter >= TimeToStartActing) {
				return;
			}
		}
		// regular processing each time step once experiment is underway
		else {
			for (int i = 0; i < itsPerTimeStep; i++) {
				rxPdu(); // receive PDU message from bacterium client
				timeStepCounter++;
				timeExperiment(); // do timed things specific to this experiment
			}
			glutPostRedisplay();
		}
	}
	else if (step || globalStep) {
		rxPdu();
		timeStepCounter++;
		timeExperiment();
		glutPostRedisplay();
		step = false;
		globalStep = false; // only the single EnvironmentApp should do this
	}
}

//////////////////////////////////////////////
// timeExperiment
// Perform certain actions at specific times.
//////////////////////////////////////////////
void EnvironmentApp::timeExperiment( void)
{
	int i, j;
	// Mop up ligands
	if (timeStepCounter == TimeToMopupLigands) {
		clearEGrid( 0 ); // clear it to all zero ligands
		eGridChanged = true;
	}
	// Create EnvironmentApp statistics
	if (timeStepCounter == TimeToCreateEaStats) {
		for (int bactIx = 0; bactIx < numCurrentBacteria; bactIx++) {
			stats->outputEnvAppHist( bactIx, bacterium[bactIx]->ligCount, NumLigands );
		}
	}
	// End experiment
	if (timeStepCounter == TimeToEndExperiment) {
		pause = true;
		globalPause = true;
	}
	// Start trial
	if (timeStepCounter == TimeToStartTrial) {
		clearEGrid( 4 ); // clear environment grid to the proper gradient
		clearBGrid(); // clear the bacterium trace
		eGridChanged = true;
		// reinitialize each bacterium proxy
		for (i = 0; i < maxBacteria; i++) {
			for (j = 0; j < NumLigands; j++) { // initialize counts to 0
				bacterium[i]->ligCount[j] = 0;
			}
			bacterium[i]->x = bacterium[i]->initialX;
			bacterium[i]->y = bacterium[i]->initialY;
			bacterium[i]->rotated = BactRot90;
		}
	}

#ifdef INCREMENT_LIGANDS
	// Inject additional ligand every n time steps
	if ((timeStepCounter < TimeToMopupLigands) && (timeStepCounter % 1000) == 0) {
		txPdu( 0, 2 ); // not a good way to do this; bipasses normal way
		// following two lines crash the program
		//NumLigands += 2;
		//clearEGrid( 5 ); // clear environment grid to the proper gradient
	}
#endif

}

//////////////////////////////////
// CallBackDisplayFunc
// Do all OpenGL graphics output.
// OpenGL (GlutMaster) callback function.
//////////////////////////////////
void EnvironmentApp::CallBackDisplayFunc( void )
{
	int col, row;
	char bactIx;
#ifdef NO_GRAPHICS
	if (eGridChanged) {
    // Make background colour a light watery blue; not really necessary
	    glClearColor( 0.8f, 0.8f, 1.0f, 0 );
		glClear ( GL_COLOR_BUFFER_BIT );
		eGridChanged = false;
	}
	return;
#endif
	if ((timeStepCounter > 0) && (timeStepCounter <= TimeToStartActing))
		return;
	// Display environment grid ("patches")
	if (eGridChanged) {
    // Make background colour a light watery blue; not really necessary
	    glClearColor( 0.8f, 0.8f, 1.0f, 0 );
		glClear ( GL_COLOR_BUFFER_BIT );
		for (row = 0; row < gridSizeY; row++) {
			for (col = 0; col < gridSizeX; col++) {
				glColor3f( 0.0f, eGrid[row*gridSizeX+col]/(1.0f*NumLigands), 0.0f);
				glRectd(unitSize*col,
					unitSize*row+unitSize,
					unitSize*col+unitSize,
					unitSize*row);
			}
		}
		eGridChanged = false;
	}
	// Display bacteria grid
	if (penStatus == PenUp) { // only draw current position
		for (bactIx = 0; bactIx < numCurrentBacteria; bactIx++) {
			if (bactIx == 0)
				glColor3f( bacterium[bactIx]->colorIntensity, 0.0f, 0.0f );
			else
				glColor3f( 0.0f, 0.0f, bacterium[bactIx]->colorIntensity );
				// adjust the color intensity so can tell which are most recent positions
				// not currently being used
				bacterium[bactIx]->colorIntensity += 0.01;
				if (bacterium[bactIx]->colorIntensity > 1.0f)
					bacterium[bactIx]->colorIntensity = 0.25f;
			glRectd(unitSize*bacterium[bactIx]->x,
				unitSize*bacterium[bactIx]->y+unitSize,
				unitSize*bacterium[bactIx]->x+unitSize,
				unitSize*bacterium[bactIx]->y);
		}
	}
	else { // penStatus == PenDown; draw history of all previous positions
		for (row = 0; row < gridSizeY; row++) {
			for (col = 0; col < gridSizeX; col++) {
				bactIx = bGrid[row*gridSizeX + col];
				if (bactIx != NullBACT) {
					if (bactIx == 0)
						glColor3f( bacterium[bactIx]->colorIntensity, 0.0f, 0.0f );
					else
						glColor3f( 0.0f, 0.0f, bacterium[bactIx]->colorIntensity );
					glRectd(unitSize*col,
						unitSize*row+unitSize,
						unitSize*col+unitSize,
						unitSize*row);
				}
			}
		}
		// do current position in a different color so can find it
		for (bactIx = 0; bactIx < numCurrentBacteria; bactIx++) {
			if (bactIx == 0)
				glColor3f( 0.0f, 0.0f, 0.0f ); // black
			else
				glColor3f( 0.0f, 0.0f, 0.0f ); // black
			glRectd(unitSize*bacterium[bactIx]->x,
				unitSize*bacterium[bactIx]->y+unitSize,
				unitSize*bacterium[bactIx]->x+unitSize,
				unitSize*bacterium[bactIx]->y);
		}
		// show initial position in another different color
		for (bactIx = 0; bactIx < numCurrentBacteria; bactIx++) {
			if (bactIx == 0)
				glColor3f( 1.0f, 1.0f, 1.0f ); // white
			else
				glColor3f( 0.5f, 0.5f, 0.5f ); // grey
			glRectd(unitSize*bacterium[bactIx]->initialX,
				unitSize*bacterium[bactIx]->initialY+unitSize*4,
				unitSize*bacterium[bactIx]->initialX+unitSize*4,
				unitSize*bacterium[bactIx]->initialY);
		}
	}
}

//////////////////////////////////////////
// CallBackReshapeFunc
// Redraw window if it's been reshaped.
// OpenGL (GlutMaster) callback function.
//////////////////////////////////////////
void EnvironmentApp::CallBackReshapeFunc(int w, int h)
{
	int row, col;
	char bactIx;
	if (Msg::infoM) printf( "width: %d  height: %d\n", w, h );
	glOrtho(0, w, 0, h, -1, 1);

	// display environment grid
	glClearColor( 0.8f, 0.8f, 1.0f, 0 );
	glClear ( GL_COLOR_BUFFER_BIT );
	for (row = 0; row < gridSizeY; row++) {
		for (col = 0; col < gridSizeX; col++) {
			glColor3f( 0.0f, eGrid[row*gridSizeX+col]/(1.0f*NumLigands), 0.0f);
			glRectd(unitSize*col,
				unitSize*row+unitSize,
				unitSize*col+unitSize,
				unitSize*row);
		}
	}
	// display bacteria grid
	for (row = 0; row < gridSizeY; row++) {
		for (col = 0; col < gridSizeX; col++) {
			bactIx = bGrid[row*gridSizeX + col];
			if (bactIx != NullBACT) {
				if (bactIx == 0)
					glColor3f( 1.0f, 0.0f, 0.0f );
				else
					glColor3f( 0.0f, 0.0f, 1.0f );
				glRectd(unitSize*col,
					unitSize*row+unitSize,
					unitSize*col+unitSize,
					unitSize*row);
			}
		}
	}
}

//////////////////////////////////////////
// CallBackKeyboardFunc
// Handle keyboard input
// OpenGL (GlutMaster) callback function.
//////////////////////////////////////////
void EnvironmentApp::CallBackKeyboardFunc(unsigned char key, int x, int y)
{
    if (key==27) // ESC key to quit
		exit(0);
	else if (key == 'p') { // pause
		if (pause) pause = false;
		else pause = true;
	}
	else if (key == 'P') { // global pause
		if (globalPause) globalPause = false;
		else globalPause = true;
	}
	else if (key == 's') // step
		step = true;
	else if (key == 'S') // globalStep
		globalStep = true;
	else if ((key == 'C') || (key == 'c')) // clear the bacterium grid
		clearBGrid();
	else if ((key >= '0') && (key <= '5')) { // clear and initialize the environment grid
		clearEGrid( key - '0' ); // a value between 0 and 5
		eGridChanged = true;
	}
	else if ((key == 'W') || (key == 'w')) { // write out statistics
		for (int bactIx = 0; bactIx < numCurrentBacteria; bactIx++) {
			stats->outputEnvAppHist( bactIx, bacterium[bactIx]->ligCount, NumLigands );
		}
	}

	else if ((key == 'h') || (key == 'H')) { // help
		printf( "Environment Application (server):\n" );
		printf( "  p pause this window/app only\n" );
		printf( "  P pause all windows/apps\n" );
		printf( "  s step this paused window/app only\n" );
		printf( "  S step all paused windows/apps\n" );
		printf( "  c clear the contents of the bacteria tracks in the display window\n" );
		printf( "  1..5 clear the environment in display window to numbered pattern\n" );
		printf( "  w write out statistics\n" );
		printf( "  h help\n" );
		printf( "  Esc exit from all windows/apps\n" );
	}
	else
		printf( "Invalid key: %c\n", key );
}

//////////////////////////////////////////////////////
// CallBackMouseFunc
// Handle mouse motion; not used for anything useful.
// OpenGL (GlutMaster) callback function.
//////////////////////////////////////////////////////
void EnvironmentApp::CallBackMouseFunc(int button, int state, int x, int y)
{
	if (state == GLUT_UP)
		if (Msg::appM) printf("x:%d y:%d\n", x, winSizeY - y);
}

////////////////////////////////////////////
// glMain
// Initialize OpenGL and some other things.
////////////////////////////////////////////
void EnvironmentApp::glMain( int argc, char *argv[] )
{
	winSizeX = gridSizeX * unitSize;
	winSizeY = gridSizeY * unitSize;
	gridSize = gridSizeX * gridSizeY;
	bGrid = new char[gridSize];
	eGrid = new int[gridSize];
	clearEGrid( ConcPatternDefault );
	clearBGrid();
    glutInitDisplayMode( GLUT_RGB | GLUT_SINGLE);
    glutInitWindowSize( winSizeX, winSizeY );
	glutMaster->CallGlutCreateWindow("Bacteria in Environment (red is WinID 1)", this);
}

///////////////////
// initialize
///////////////////
void EnvironmentApp::initialize( char *configFileName )
{
	// Read values from configuration file
	readConfigFromFile( configFileName );
}

////////////////////////////////////////////
// readConfigFromFile
// Read configuration values from a file.
////////////////////////////////////////////
void EnvironmentApp::readConfigFromFile( char *fileName )
{
	FILE *fp;
	char *mode = "r";
	const int sizeBuff = 100;
	char buffer[sizeBuff];
	int intVal, msgVal1, msgVal2, msgVal3;
	bool showParms = false;

	if ((fp = fopen(fileName, mode)) == NULL) {
		if (Msg::errorM) printf( "Can't open file: %s in %s mode\n", fileName, mode);
		exit(0);
	}
	else {
		// printf("Opened file: %s in %s mode\n", fileName, mode);
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%d", &intVal ) != 1) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (intVal == 1)
				showParms = true;
		}
		// Msg::appM Msg::infoM Msg::errorM
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%d %d %d", &msgVal1, &msgVal2, &msgVal3 ) != 3) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			// appM
			if (msgVal1 == 0)
				Msg::appM = false;
			else
				Msg::appM = true;
			// infoM
			if (msgVal2 == 0)
				Msg::infoM = false;
			else
				Msg::infoM = true;
			// errorM
			if (msgVal3 == 0)
				Msg::errorM = false;
			else
				Msg::errorM = true;
			if (showParms) {
				printf( "Msg::appM   : %d\n", Msg::appM );
				printf( "Msg::infoM  : %d\n", Msg::infoM );
				printf( "Msg::errorM : %d\n", Msg::errorM );
			}
		}
		// gridSizeX gridSizeY
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%d %d", &gridSizeX, &gridSizeY ) != 2) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (showParms) {
				printf( "gridSizeX : %d\n", gridSizeX );
				printf( "gridSizeY : %d\n", gridSizeY );
			}
		}
		// unitSize
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%d", &unitSize ) != 1) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (showParms) {
				printf( "unitSize : %d\n", unitSize );
			}
		}
		// itsPerTimeStep
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%d", &itsPerTimeStep ) != 1) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (showParms) {
				printf( "itsPerTimeStep : %d\n", itsPerTimeStep );
			}
		}
		// NumLigands
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%d", &NumLigands ) != 1) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (showParms) {
				printf( "NumLigands : %d\n", NumLigands );
			}
		}
		// ConcPatternDefault
		if (fgets(buffer, sizeBuff, fp) != NULL) {
			if (sscanf( buffer, "%d", &ConcPatternDefault ) != 1) {
				if (Msg::errorM) printf("Application: error reading from %s\n", fileName);
				exit(0);
			}
			if (showParms) {
				printf( "ConcPatternDefault : %d\n", ConcPatternDefault );
			}
		}
		fclose(fp);
	}
}

////////////////////////////////////////////////////////////////////////////////////
// setQ
// Set a pair of queues, and also establish bacterium corresponding to those queue.
////////////////////////////////////////////////////////////////////////////////////
void EnvironmentApp::setQ( Queue *iq, Queue *oq )
{
	inQ[numCurrentBacteria]  = iq;
	outQ[numCurrentBacteria] = oq;
	numCurrentBacteria++;
}

//////////////////////////////////////////////////
// rxPdu
// Receive PDUs from all Qs. (a PDU is a message)
//////////////////////////////////////////////////
void EnvironmentApp::rxPdu( void )
{
	BactEnvPdu *bep;
	// process queue of each bacterium
	for (int bactIx = 0; bactIx < numCurrentBacteria; bactIx++) {
		while ((bep = (BactEnvPdu*)inQ[bactIx]->dequeue()) != Q_EMPTY) {
			if (bep->pduType == RunBPT) {
				// fd 1
				run( bactIx );
			}
			else { // == TumbleBPT
				// right random 360
				tumble( bactIx );
			}
			delete bep; // all done with it so delete what client created
		}
	}
}

//////////////////////////////
// txPdu
// Transmit one PDU to one Q.
//////////////////////////////
void EnvironmentApp::txPdu( int bactIx, int pVal )
{
	BactEnvPdu *bep;
	bep = new BactEnvPdu();
	bep->pduType = GradientBPT;
	bep->pduVal  = pVal;
	if (outQ[bactIx]->enqueue( bep ) == Q_FULL)
		delete bep;
}

//////////////////////////////////
// run
// Handle bacterium run behavior.
//////////////////////////////////
void EnvironmentApp::run( int bactIx )
{
	// fd 1
	int pduVal;
	ligandConcBefore = ligandConcAfter;
	if (timeStepCounter < TimeToMopupLigands) // don't collect stats once ligands are gone
		bacterium[bactIx]->ligCount[ligandConcBefore]++; // stats
	// bacterium is facing in one of four cardinal directions
	switch (bacterium[bactIx]->rotated) {
	case BactRot0:
		bacterium[bactIx]->y++;
		if (bacterium[bactIx]->y >= gridSizeY)
			bacterium[bactIx]->y = 0;
		break;
	case BactRot90:
		bacterium[bactIx]->x++;
		if (bacterium[bactIx]->x >= gridSizeX)
			bacterium[bactIx]->x = 0;
		break;
	case BactRot180:
		bacterium[bactIx]->y--;
		if (bacterium[bactIx]->y < 0)
			bacterium[bactIx]->y = gridSizeY-1;
		break;
	case BactRot270:
		bacterium[bactIx]->x--;
		if (bacterium[bactIx]->x < 0)
			bacterium[bactIx]->x = gridSizeX-1;
		break;
	default:
		break;
	}
	// save in bacterium grid
	bGrid[bacterium[bactIx]->y * gridSizeX + bacterium[bactIx]->x] = bactIx;
	// calculate difference in lignd concentration since last time step
	ligandConcAfter = eGrid[bacterium[bactIx]->y * gridSizeX + bacterium[bactIx]->x];
	pduVal = ligandConcAfter - ligandConcBefore;
	// send updated ligand concentration to bacterium client
#ifdef CONTROL_GROUP
	if ((pduVal == 1) || (pduVal == -1))
		txPdu( bactIx, 0 ); // don't tell client about gradient
	else
		txPdu( bactIx, pduVal ); // only tell client when ligands initially added
#else // experimental group
	txPdu( bactIx, pduVal );
#endif
}

//////////////////////////////////////
// tumble
// Handle bacterdium tumble behavior.
//////////////////////////////////////
void EnvironmentApp::tumble( int bactIx )
{
	bacterium[bactIx]->rotated += BactRot90;
	if (bacterium[bactIx]->rotated > BactRot270)
		bacterium[bactIx]->rotated = BactRot0;
	// collect stats
	if (timeStepCounter < TimeToMopupLigands) // don't collect stats once ligands are gone
		bacterium[bactIx]->ligCount[ligandConcAfter]++; // stats
}

/////////////////////////////////////////////
// clearBGrid
// Clear the contents of the bacterium grid.
/////////////////////////////////////////////
void EnvironmentApp::clearBGrid( void )
{
	for (int i = 0; i < gridSize; i++) {
		bGrid[i] = NullBACT;
	}
}

////////////////////////////////////////////////////////////////////////
// clearEGrid
// Create the contents of the environment grid.
// This will appear in the graphics window as various colored patterns.
////////////////////////////////////////////////////////////////////////
void EnvironmentApp::clearEGrid( int concPattern )
{
	int row, col;
	int ligandConc = 0;
	int inc = 1;
	int signMult = 1;

	switch (concPattern) {
	case 0: // no gradient; ligand concetration = 0
		ligandConc = 0;
		for (row = 0; row < gridSizeY; row++) {
			for (col = 0; col < gridSizeX; col++) {
				eGrid[row*gridSizeX+col] = ligandConc;
			}
		}
		break;
	case 1: // smooth vertical gradient
		for (row = 0; row < gridSizeY; row++) {
			for (col = 0; col < gridSizeX; col++) {
				eGrid[row*gridSizeX+col] = ligandConc;
			}
			if ((row >= ligandStartRow) && (row < ligandEndRow))
				ligandConc += inc;
		}
		break;
	case 2: // more abrupt vertical gradient
		for (row = 0; row < gridSizeY; row++) {
			for (col = 0; col < gridSizeX; col++) {
				eGrid[row*gridSizeX+col] = ligandConc;
			}
			if ((row >= ligandStartRow) && (row < ligandEndRow))
				if ((row % 10) == 0)
					ligandConc += inc * 10.0;
		}
		break;
	case 3: // one solid ligand block
		for (row = 0; row < gridSizeY; row++) {
			for (col = 0; col < gridSizeX; col++) {
				eGrid[row*gridSizeX+col] = ligandConc;
			}
		}
		for (row = ligandStartRow+31; row < ligandStartRow+231; row++) {
			for (col = 0; col < gridSizeX; col++) {
				eGrid[row*gridSizeX+col] = ligandConc + NumLigands;
			}
		}
		break;
	case 4: // smoothly varying vertical gradient
		for (row = 0; row < gridSizeY; row++) {
			for (col = 0; col < gridSizeX; col++) {
				eGrid[row*gridSizeX+col] = ligandConc;
			}
			ligandConc += inc * signMult;
			if (ligandConc < 0) {
				ligandConc = 0;
				signMult = 1;
			}
			else if (ligandConc >= NumLigands) {
				ligandConc = NumLigands - inc;
				signMult = -1;
			}
		}
		// confirmation of ranges
		//for (row = 0; row < gridSizeY; row++) {
		//	printf( "|%d|", eGrid[row*gridSizeX] );
		//}
		break;
	case 5: // no gradient; continuous concetration of ligand
		ligandConc = NumLigands - 1; // offset from 0, so has to be - 1
		for (row = 0; row < gridSizeY; row++) {
			for (col = 0; col < gridSizeX; col++) {
				eGrid[row*gridSizeX+col] = ligandConc;
			}
		}
		break;
	case 6: // no gradient; continuous concetration of ligand; NumLigands * 0.5
		ligandConc = (NumLigands * 0.5) - 1;
		for (row = 0; row < gridSizeY; row++) {
			for (col = 0; col < gridSizeX; col++) {
				eGrid[row*gridSizeX+col] = ligandConc;
			}
		}
		break;
	default:
		break;
	}
}
