/**************************************************************************
EASY MSc Dissertation - Project
Ken Webb
August 8, 2004
EnvironmentApp.h: interface for the EnvironmentApp class.

This class represents the environment that surrounds one or more bacterium.
It is optional, and is only used to visualize bacteria moving on a grid in
a 2D environment.
**************************************************************************/

#if !defined(AFX_ENVIRONMENTAPP_H__B6D6AFC0_7F08_4CE5_BCA2_0061F99A510A__INCLUDED_)
#define AFX_ENVIRONMENTAPP_H__B6D6AFC0_7F08_4CE5_BCA2_0061F99A510A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Application.h"
#include "Msg.h"
#include "Rndm.h"
#include "GlutMaster.h"
#include "BacteriumProxy.h"
#include "Queue.h"
#include "Statistics.h"
#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>

// Rotations of the bacterium
const int BactRot0     = 0; // default
const int BactRot90   = 90;
const int BactRot180 = 180;
const int BactRot270 = 270;

// Pen up/down
const int PenUp   = 1;
const int PenDown = 2;

const char NullBACT = -1; // no bacteria have been in the grid yet
const float NullLIG  = 0.8; // no ligand in environment

// EnvironmentApp
class EnvironmentApp : public Application  
{
public:
	EnvironmentApp( GlutMaster* gm, Statistics* st );
	virtual ~EnvironmentApp();

	// inherited from Application superclass
	void CallBackDisplayFunc();
	void CallBackKeyboardFunc(unsigned char key, int x, int y);
	void CallBackMouseFunc(int button, int state, int x, int y);
	void CallBackReshapeFunc(int w, int h);   
	void CallBackTimerFunc(int value);
	void initialize( char *configFileName );

	void setQ( Queue *iq, Queue *oq ); // specify message queue(s)
	void glMain( int argc, char *argv[] ); // initialize GL

protected:
	GlutMaster* glutMaster; // interface to OpenGL; allows multiple OpenGL windows

	// Statistics
	Statistics *stats; // ptr to the single statistics instance

	// OpenGL window size
	int winSizeX;
	int winSizeY;

	// Number of screen coordinate "pixels" in a unit
	int unitSize; // default: 1

	// Time step
	int timeStepCounter; // counter
	int itsPerTimeStep; // number of iterations per time step

	// Pause
	bool pause;
	bool step;

	// flag to specify special actions during first call to a function
	bool firstTime;

	// Pen status (up or down when drawing)
	int penStatus;

	// need to be able to handle multiple in and out queues; multiple clients
	Queue **outQ; // queues to send BactEnvPdu messages to a bacterium client
	Queue **inQ;  // queues to receive BactEnvPdu messages from a bacterium client

	// grid
	int gridSizeX;
	int gridSizeY;
	int gridSize; // gridSizeX * gridSizeY
	char *bGrid; // bacterium grid contains bacterium IDs (typically 0 to 9)
	int *eGrid; // environment grid
	bool eGridChanged; // has the eGrid changed?

	// Bacteria proxies
	BacteriumProxy **bacterium; // ptr to array of pointers to BacteriumProxy
	int maxBacteria; // maximum number of bacteria in the application
	int numCurrentBacteria; // current number of bacteria in the app

	// current ligand concentrations
	int NumLigands; // number of ligands; 20 times value in BacteriumApp
	int ligandConcBefore; // at start of a time step
	int ligandConcAfter;  // at end of a time step
	int ConcPatternDefault; // ligand concentration pattern

	void readConfigFromFile( char *fileName ); // read from config file
	void initSim( int tVal ); // initialize
	void actSim( int tVal ); // regular action at each time step
	void rxPdu( void ); // receive a PDU message from BacteriumApp
	void txPdu( int bactIx, int pVal ); // transmit a PDU message to BacteriumApp
	void run( int bactIx ); // handle a received RunBPT PDU
	void tumble( int bactIx ); // handle a received TumbleBPT PDU
	void clearBGrid( void ); // clear the bacterium grid
	void clearEGrid( int concPattern ); // clear and initialize the environment grid
	void timeExperiment( void); // perform certain actions at specific times
};

#endif // !defined(AFX_ENVIRONMENTAPP_H__B6D6AFC0_7F08_4CE5_BCA2_0061F99A510A__INCLUDED_)
