////////////////////////////////////////////////////////////////
//                                                            //
// glutMaster.c++                                             //
// beta version 0.3 - 9/9/97)                                 //
//                                                            //
// George Stetten and Korin Crawford                          //
// copyright given to the public domain                       //
//                                                            //
// Please email comments to stetten@acpub.duke.edu,           //
//                                                            //
////////////////////////////////////////////////////////////////
// adapted by Ken Webb (KSW)

#include "glutMaster.h"
#include "glutWindow.h"
#include <stdio.h>
                                                       
GlutWindow * viewPorts[MAX_NUMBER_OF_WINDOWS]; 

int GlutMaster::currentIdleWindow   = 0;
int GlutMaster::idleFunctionEnabled = 0;
int GlutMaster::timerStepMsecs = 0; // KSW
int GlutMaster::timerValue = 0; // KSW


GlutMaster::GlutMaster(){

	// initialize viewPorts to all NULL		KSW
	for (int i = 0; i < MAX_NUMBER_OF_WINDOWS; i++)
		viewPorts[i] = NULL;

   // Create dummy variables 

   char * dummy_argv[1];
   dummy_argv[0] = "run";
   int dummy_argc = 1;

   // Initialize GLUT

   glutInit(&dummy_argc, dummy_argv);
}

GlutMaster::~GlutMaster(){

}
 
void GlutMaster::CallBackDisplayFunc(void){
   int windowID;

   for (windowID = 0; windowID < MAX_NUMBER_OF_WINDOWS; windowID++) {
	   if (viewPorts[windowID] != NULL) {
		   glutSetWindow(windowID);
		   viewPorts[windowID]->CallBackDisplayFunc();
	   }
   }
}

void GlutMaster::CallBackIdleFunc(void){

   if(idleFunctionEnabled && currentIdleWindow){
      glutSetWindow(currentIdleWindow);
      viewPorts[currentIdleWindow]->CallBackIdleFunc();
   }
}
 
void GlutMaster::CallBackKeyboardFunc(unsigned char key, int x, int y){

   int windowID = glutGetWindow();
   viewPorts[windowID]->CallBackKeyboardFunc(key, x, y);
}

void GlutMaster::CallBackMotionFunc(int x, int y){

   int windowID = glutGetWindow();
   viewPorts[windowID]->CallBackMotionFunc(x, y);
}

void GlutMaster::CallBackMouseFunc(int button, int state, int x, int y){

   int windowID = glutGetWindow();
   viewPorts[windowID]->CallBackMouseFunc(button, state, x, y);
   currentIdleWindow = glutGetWindow(); // KSW
}

void GlutMaster::CallBackPassiveMotionFunc(int x, int y){

   int windowID = glutGetWindow();
   viewPorts[windowID]->CallBackPassiveMotionFunc(x, y);
}

void GlutMaster::CallBackReshapeFunc(int w, int h){

   int windowID = glutGetWindow();
   viewPorts[windowID]->CallBackReshapeFunc(w, h);
}

void GlutMaster::CallBackSpecialFunc(int key, int x, int y){

   int windowID = glutGetWindow();
   viewPorts[windowID]->CallBackSpecialFunc(key, x, y);
}   

void GlutMaster::CallBackTimerFunc(int value) // KSW
{
   int windowID;

   // I need to call each open window
   for (windowID = 0; windowID < MAX_NUMBER_OF_WINDOWS; windowID++) {
	   if (viewPorts[windowID] != NULL)
		   viewPorts[windowID]->CallBackTimerFunc(value);
   }

   glutTimerFunc(timerStepMsecs, CallBackTimerFunc, timerValue);
}

void GlutMaster::CallBackVisibilityFunc(int visible){

   int windowID = glutGetWindow();
   viewPorts[windowID]->CallBackVisibilityFunc(visible);
}

void GlutMaster::CallGlutCreateWindow(char * setTitle, GlutWindow * glutWindow){

   // Open new window, record its windowID , 

   int windowID = glutCreateWindow(setTitle);

   glutWindow->SetWindowID(windowID);

   // Store the address of new window in global array 
   // so GlutMaster can send events to propoer callback functions.

   viewPorts[windowID] = glutWindow;

   // Hand address of universal static callback functions to Glut.
   // This must be for each new window, even though the address are constant.

   // Those commented out are not used in this application.
   glutDisplayFunc(CallBackDisplayFunc);
   //glutIdleFunc(CallBackIdleFunc); 
   glutKeyboardFunc(CallBackKeyboardFunc);
   glutMouseFunc(CallBackMouseFunc);
   //glutMotionFunc(CallBackMotionFunc);
   //glutPassiveMotionFunc(CallBackPassiveMotionFunc);
   glutReshapeFunc(CallBackReshapeFunc);
   //glutVisibilityFunc(CallBackVisibilityFunc);
}

void GlutMaster::CallGlutMainLoop(void){

   glutMainLoop();
}
                              
void GlutMaster::DisableIdleFunction(void){

   idleFunctionEnabled = 0;
}

void GlutMaster::EnableIdleFunction(void){

   idleFunctionEnabled = 1;
}

int GlutMaster::IdleFunctionEnabled(void){

   // Is idle function enabled?

   return(idleFunctionEnabled);
}

int GlutMaster::IdleSetToCurrentWindow(void){

   // Is current idle window same as current window?

   return( currentIdleWindow == glutGetWindow() );
}

void GlutMaster::SetIdleToCurrentWindow(void){

   currentIdleWindow = glutGetWindow();
}

void GlutMaster::SetTimerFunc( int initMsecs, int stepMsecs, int value ) // KSW
{
   timerStepMsecs = stepMsecs;
   timerValue = value;
   glutTimerFunc(initMsecs, CallBackTimerFunc, value);
}
