/**************************************************************************
EASY MSc Dissertation - Project
Ken Webb
August 4, 2004
Grid.h: interface for the Grid class.

Every instance of BacteriumApp has a grid instance that stores all the polymers,
and represents all the monomers on a set of 2D grids.
**************************************************************************/

#if !defined(AFX_GRID_H__4766CB43_9EDD_423E_8AA6_8BC5119740C9__INCLUDED_)
#define AFX_GRID_H__4766CB43_9EDD_423E_8AA6_8BC5119740C9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Polymer.h"
#include "Monomer.h"
#include "Probabilities.h"
#include "Statistics.h"

// Because of the size of the various grids, the size of each element in a 
// grid is important. "short unsigned" is only 16 bits as opposed to "int" which is 32.
typedef short unsigned gint;
// max value of short unsigned; may be platform dependent
const gint NullPolyID = 65535;

// Grid Layers
// There are four different grid layers, each containing a different type of polymer/monomer.
typedef enum GridLayerEnum {
	NullLY = 0,
	MembraneLY = 1,
	Protein1LY,
	Protein2LY,
	SmallMolLY
} GridLayerEnum;

// Grid
class Grid  
{
public:
	Grid(); // don't use this constructor
	Grid( int nRows, int nCols, int maxPoly, Probabilities* pr, Statistics *st);
	virtual ~Grid();
	// Get and set functions
	int getNumRows( void );
	int getNumCols( void );
	int getGridSize( void );
	int getNumCurrentPolymers( void );
	gint* getLayerMono( int gridLayer );
	gint* getLayerPoly( int gridLayer );
	void addPolymer( Polymer* pol );
	Polymer* getPolymer( int ix );

	void act( void ); // call act() functions for all polymer instances
	void setNoMoreRules( void ); // set special rule in each polymer, for performance reasons
	void inventory( int clientNum, Monomers *mn ); // save an inventory file with info on all polymers
	void materializeGhosts( int num2Materialize ); // make a given number of ligands become part of the sim
	void mopUpLigands( void ); // return all ligands to ghost status
	void mopUpLigands( int num2MopUp ); // return some ligands to ghost status
	Statistics *stats; // ptr to the single statistics object
protected:
	int numRows; // number of rows in the overall grid
	int numCols; // number of columns
	int gridSize; // numRows * numCols
	bool doActions; // whether act() should call pol->act()
	Probabilities* prob; // ptr to the single probabilities object

	// Polymers
	Polymer **polymer; // ptr to array of pointers to Polymer
	int maxPolymers; // maximum number of polymers in the application
	int numCurrentPolymers; // current number of polymers in the app

	// Membrane Layer (lipid bilayers)
	gint *LMembraneMonoId; // monomer IDs
	gint *LMembranePolyId; // polymer IDs
	// Protein Layer 1 ("red" proteins; those forming permanent protein complexes)
	gint *LProtein1MonoId; // monomer IDs
	gint *LProtein1PolyId; // polymer IDs
	// Protein Layer 2 ("brown" proteins; those that bind to proteins in Protein Layer 1)
	gint *LProtein2MonoId; // monomer IDs
	gint *LProtein2PolyId; // polymer IDs
	// Small Molecule Layer (ligands, ATP, methanol)
	gint *LSmallMolMonoId; // monomer IDs
	gint *LSmallMolPolyId; // polymer IDs

	void initGrids( void ); // initialize all grids to default values
	void placeInGrid( Polymer* pol, int ix ); // place a polymer in a grid
	void handleEntropy( Polymer* pol1 ); // break bindings using probabilities
	void adjustBindings( Polymer* pol1 ); // adjust bindings that may now be broken
	// check if two monomers match
	bool matchM_M( Polymer* pol1, Polymer* pol2, MonomerTypeEnum mon1, MonomerTypeEnum mon2 );
};

#endif // !defined(AFX_GRID_H__4766CB43_9EDD_423E_8AA6_8BC5119740C9__INCLUDED_)
