/**************************************************************************
EASY MSc Dissertation - Project
Ken Webb
August 4, 2004
LipidBilayer.h: interface for the LipidBilayer class.

A lipid bilayer is a type of polymer. It encloses proteins and other polymers,
is composed of lipid monomers, and in many ways represents a bacterium as
a whole.
It sums together runs and tumbles information from the several motor complexes,
and processes this in various ways.
**************************************************************************/

#if !defined(AFX_LIPIDBILAYER_H__913B1A27_DE7E_4B6F_9121_9958D947E330__INCLUDED_)
#define AFX_LIPIDBILAYER_H__913B1A27_DE7E_4B6F_9121_9958D947E330__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Polymer.h"
#include "Grid.h"
#include "Queue.h"

// uncomment this to do motor calibration
//#define MOTOR_CALIBRATION 1

// LipidBilayer
class LipidBilayer : public Polymer  
{
public:
	LipidBilayer();
	LipidBilayer( Grid *g, int xx, int yy, int r, int f, int gl, Polymer* c, bool a, int h, int w );
	virtual ~LipidBilayer();
	// set the rectangular shape of the lipid monomers
	void setShape( int nRows, int nCols );
	void act( void ); // do an action every time step
	void move( void ); // move every time step; LipidBilayer makes no movement

	void incCurrentCCW( void ); // increment the current counterClockWise CCW counter
	double getTotalRuns( void ); // return the total number of runs this time period
	double getTotalTumbles( void ); // return the total number of tumbles this time period
	// initialize details on handling run and tumble information
	void setProcessRunsTumbles( bool prt, Queue *q, int winID );

	void inventory( FILE *fp, Monomers *mn ); // save inventory info

protected:
	Grid *grid; // the single grid object that contains all the polymers
	Queue *outQ; // queue to send BactEnvPdu messages to EnvironmentApp server
	int windowID; // ID of this OpenGL window

	// Alphabet of valid constituent monomers
	static int alphabet[];
	static int lenAlphabet;

	// a lipidBilayer represents an entire E Coli bacterium
	// keep track of how the motors altogether are moving, so can know which way eColi should move
	double totalRuns; // total number of run timesteps since beginning of simulation
	double totalTumbles; // total number of tumble timesteps
	int currentCCW; // number of CCW movements this time step
	int timeStepCounter; // has its own counter; too hard to get at global counter
	bool processRunsTumbles; // should this LipidBilayer process runs and tumbles
	void plotPropRuns( double numRuns, double numTumbles );

	// Stats
	void doStats( void );

	// Motor Calibration
#ifdef MOTOR_CALIBRATION
	// Motor calibration
	double totalRunsCalib; // total runs for motor calibration
	double totalTumblesCalib; // total tumbles for motor calibration
	int timeStepCounterCalib;
#endif
};

#endif // !defined(AFX_LIPIDBILAYER_H__913B1A27_DE7E_4B6F_9121_9958D947E330__INCLUDED_)
