/**************************************************************************
EASY MSc Dissertation - Project
Ken Webb
August 5, 2004
Monomer.cpp: implementation of the Monomer class.

**************************************************************************/

#include "Monomer.h"

// Table describing each monomer type
// red, green, blue values are needed for OpenGL graphics
const MonomerType Monomers::mnTypeTab[SizeOfMonomerEnumMN] = {
/*
id       red   green blue code  name
-------  ----  ----  ---- ----  -------
*/
NullMN,  0.0f, 0.0f, 0.0f, '-', "Null",
WaterMN, 0.5f, 0.5f, 1.0f, '_', "Water",

// Amino Acids
AlaMN, 0.46f, 0.0f, 0.0f, 'A', "Ala",
CysMN, 0.49f, 0.0f, 0.0f, 'C', "Cys",
AspMN, 0.52f, 0.0f, 0.0f, 'D', "Asp",
GluMN, 0.30f, 0.0f, 0.0f, 'E', "Glu",
PheMN, 0.58f, 0.0f, 0.0f, 'F', "Phe",
GlyMN, 1.00f, 0.5f, 0.5f, 'G', "Gly",
HisMN, 0.61f, 0.0f, 0.0f, 'H', "His",
IleMN, 0.64f, 0.0f, 0.0f, 'I', "Ile",
LysMN, 0.67f, 0.0f, 0.0f, 'K', "Lys",
LeuMN, 0.70f, 0.0f, 0.0f, 'L', "Leu",
MetMN, 0.73f, 0.0f, 0.0f, 'M', "Met",
AsnMN, 0.76f, 0.0f, 0.0f, 'N', "Asn",
ProMN, 0.79f, 0.0f, 0.0f, 'P', "Pro",
GlnMN, 0.82f, 0.0f, 0.0f, 'Q', "Gln",
ArgMN, 0.85f, 0.0f, 0.0f, 'R', "Arg",
SerMN, 0.88f, 0.0f, 0.0f, 'S', "Ser",
ThrMN, 0.91f, 0.0f, 0.0f, 'T', "Thr",
ValMN, 0.94f, 0.0f, 0.0f, 'V', "Val",
TrpMN, 0.97f, 0.0f, 0.0f, 'W', "Trp",
TyrMN, 1.00f, 0.0f, 0.0f, 'Y', "Tyr",

// Phosphorylated Amino Acids
HisPhosMN, 0.61f, 0.0f, 0.5f, 'h', "HisPhos",
SerPhosMN, 0.88f, 0.0f, 0.5f, 's', "SerPhos",
ThrPhosMN, 0.91f, 0.0f, 0.5f, 't', "ThrPhos",
TyrPhosMN, 1.00f, 0.0f, 0.5f, 'y', "TyrPhos",

// Methylated Amino Acids
GluMeth1MN, 0.41f, 0.0f, 0.4f, '1', "GluMeth1",
GluMeth2MN, 0.68f, 0.0f, 0.4f, '2', "GluMeth2",
GluMeth3MN, 0.71f, 0.0f, 0.4f, '3', "GluMeth3",
GluMeth4MN, 0.80f, 0.0f, 0.4f, '4', "GluMeth4",

// Nucleic Acid (DNA/RNA) Bases
GmpMN, 0.0f, 0.0f, 0.0f, 'G', "G",
CmpMN, 0.0f, 0.0f, 0.0f, 'C', "C",
AmpMN, 0.0f, 0.0f, 0.0f, 'A', "A",
TmpMN, 0.0f, 0.0f, 0.0f, 'T', "T",
UmpMN, 0.0f, 0.0f, 0.0f, 'U', "U",

// Lipid Monomers (yellowish)
Lip1MN, 1.0f, 1.0f, 0.0f, '1', "Lipid1",
Lip2MN, 0.9f, 0.9f, 0.0f, '2', "Lipid2",

// Ions
Ion1MN, 0.0f, 0.0f, 0.0f, 'i', "Ion1",
Ion2MN, 0.0f, 0.0f, 0.0f, 'i', "Ion2",

// Other Small Molecules
SmA_MN, 0.0f, 0.46f, 0.0f, 'A', "SmA",
SmC_MN, 0.0f, 0.49f, 0.0f, 'C', "SmC",
SmD_MN, 0.0f, 0.52f, 0.0f, 'D', "SmD",
SmE_MN, 0.0f, 0.55f, 0.0f, 'E', "SmE",
SmF_MN, 0.0f, 0.58f, 0.0f, 'F', "SmF",
SmG_MN, 0.5f, 1.00f, 0.5f, 'G', "SmG",
SmH_MN, 0.0f, 0.61f, 0.0f, 'H', "SmH",
SmI_MN, 0.0f, 0.64f, 0.0f, 'I', "SmI",
SmK_MN, 0.0f, 0.67f, 0.0f, 'K', "SmK",
SmL_MN, 0.0f, 0.70f, 0.0f, 'L', "SmL",
SmM_MN, 0.0f, 0.73f, 0.0f, 'M', "SmM",
SmN_MN, 0.0f, 0.76f, 0.0f, 'N', "SmN",
SmP_MN, 0.0f, 0.79f, 0.0f, 'P', "SmP",
SmQ_MN, 0.0f, 0.82f, 0.0f, 'Q', "SmQ",
SmR_MN, 0.0f, 0.85f, 0.0f, 'R', "SmR",
SmS_MN, 0.0f, 0.88f, 0.0f, 'S', "SmS",
SmT_MN, 0.0f, 0.91f, 0.0f, 'T', "SmT",
SmV_MN, 0.0f, 0.94f, 0.0f, 'V', "SmV",
SmW_MN, 0.0f, 0.97f, 0.0f, 'W', "SmW",
SmY_MN, 0.0f, 1.00f, 0.0f, 'Y', "SmY",

// Adenosine phosphates (small molecule monomers)
AdenosineMN,       0.0f, 0.5f, 0.5f, 'a', "Adenosine",
PhosphorylGroupMN, 0.5f, 0.0f, 0.5f, 'p', "PhosphorylGroup",

// monomers of the Methanol polymer
MethBaseMN,        0.0f, 0.4f, 0.4f, 'm', "MethBase",
MethylGroupMN,     0.4f, 0.0f, 0.4f, 'm', "MethylGroup",
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Monomers::Monomers()
{

}

Monomers::~Monomers()
{

}
