/**************************************************************************
EASY MSc Dissertation - Project
Ken Webb
August 4, 2004
Monomer.h: interface for the Monomer class.

Monomers are represented as enumerated numbers rather than as class instances.
Each polymer contains an array with a sequence of monomers.
Each enumerated monomer type has an entry in the global MonomerType table,
that gives its ID, color, single-letter code, and name.
**************************************************************************/

#if !defined(AFX_MONOMER_H__DB5A6772_E3D7_4C0A_8FCA_A5E555E4340C__INCLUDED_)
#define AFX_MONOMER_H__DB5A6772_E3D7_4C0A_8FCA_A5E555E4340C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// Monomer Types (MN)
// defines the valid monomer types, and the number of entries in the monomer type table
typedef enum MonomerTypeEnum {
NullMN = 0, // end of string
// Water; fills all the holes
WaterMN,

// Amino Acids
AlaMN, // A Alanine
CysMN, // C Cysteine
AspMN, // D AsparticAcid
GluMN, // E GlutamicAcid
PheMN, // F Phenylalanine
GlyMN, // G Glycine is the most flexible amino acid
HisMN, // H Histidine
IleMN, // I Isoleucine
LysMN, // K Lysine
LeuMN, // L Leucine
MetMN, // M Methionine
AsnMN, // N Asparagine
ProMN, // P Proline
GlnMN, // Q Glutamine
ArgMN, // R Arginine
SerMN, // S Serine
ThrMN, // T Threonine
ValMN, // V Valine
TrpMN, // W Tryptophan
TyrMN, // Y Tyrosine

// Phosphorylated Amino Acids (there are only three, plus His)
HisPhosMN, // phosphorylated Histidine
SerPhosMN, // phosphorylated Serine
ThrPhosMN, // phosphorylated Threonine
TyrPhosMN, // phosphorylated Tyrosine

// Methylated Amino Acids (with 1,2,3 or 4 methyl groups attached)
GluMeth1MN,
GluMeth2MN,
GluMeth3MN,
GluMeth4MN,

// Nucleic Acid (DNA/RNA) Bases
GmpMN, // G
CmpMN, // C
AmpMN, // A
TmpMN, // T
UmpMN, // U

// Lipid Monomers
Lip1MN,
Lip2MN,

// Ions
Ion1MN,
Ion2MN,

// Other Small Molecule monomers
// One to match each amino acid
// This is my simplified way of doing small molecules
SmA_MN,
SmC_MN,
SmD_MN,
SmE_MN,
SmF_MN,
SmG_MN,
SmH_MN,
SmI_MN,
SmK_MN,
SmL_MN,
SmM_MN,
SmN_MN,
SmP_MN,
SmQ_MN,
SmR_MN,
SmS_MN,
SmT_MN,
SmV_MN,
SmW_MN,
SmY_MN,

// Adenosine phosphates (small molecule monomers)
// Adenosine monophosphate (AMP) = AdenosineMN + 1 PhosphorylGroupMN
// Adenosine diphosphate   (ADP) = AdenosineMN + 2 PhosphorylGroupMN
// Adenosine triphosphate  (ATP) = AdenosineMN + 3 PhosphorylGroupMN
AdenosineMN,
PhosphorylGroupMN,

// monomers of the Methanol polymer
MethBaseMN, // permanent monomer in the Methanol polymer
MethylGroupMN, // monomer that gets transferred to other polymers

// Size of enum; use to determine table size
SizeOfMonomerEnumMN
} MonomerTypeEnum;

// Groups; wild cards
const int WildCardMN = 1001; // #
const int WildCardNoGlyMN = 1002; // *
const int HydrophobicMN = 1003; // h
const int ChargedMN = 1004; // c
const int PolarMN = 1005;  // p

// Special "monomers"
const int TurnMN       = GlyMN; // G
const int MatchWaterMN = TrpMN; // W or w; Tryptophan
const int DelimitMN    = GlnMN; // Q or q; Glutamine

// 
struct MonomerType
{
	int id;
	// RGB colors
	float colorRed;
	float colorGreen;
	float colorBlue;
	// One-letter code
	char code;
	// Name of the monomer
	char* name;
};

// Encapsulates the table of Monomer info
class Monomers
{
public:
	Monomers();
	virtual ~Monomers();
	static const MonomerType* get(int mnTypeID)
	{
		return &mnTypeTab[mnTypeID];
	}
private:
	static const MonomerType mnTypeTab[SizeOfMonomerEnumMN];
};

#endif // !defined(AFX_MONOMER_H__DB5A6772_E3D7_4C0A_8FCA_A5E555E4340C__INCLUDED_)
