/**************************************************************************
EASY MSc Dissertation - Project
Ken Webb
August 4, 2004
Probabilities.h: interface for the Probabilities class.

Probabilities of rules and other protein (polymer) actions happening
during any individual time step.
Should be able to evolve these in a GA.
An instance of this class is a complete "genome".
All probabilities:
  - should be public data members of this class
  - are of type int
  - will be passed to all protein instances
  - are a proportion out of 10000
    ex: 50 means 50 times out of 10000
Only need one instance of this class for each BacteriumApp instance.
  - all proteins within an App are from the same genome
Each protein should be passed a ptr to this single instance.
If there are 2+ instances of BacteriumApp running, each has its own instance.
**************************************************************************/

#if !defined(AFX_PROBABILITIES_H__EFF44007_D82A_4FAF_80E3_09F56844BED9__INCLUDED_)
#define AFX_PROBABILITIES_H__EFF44007_D82A_4FAF_80E3_09F56844BED9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// Probability identifiers
// these identify a slot in the probability array
typedef enum ProbIdEnum {

	// probability that small molecule ligands will unbind from TMP protein
	ProbLigandUnbindPRI,

	// Protein::doRule_AutoPhosphorylation
	ProbCheAAutoPhosPRI, // NOT out of 10000

	// Protein::doRule_AutoDephosphorylation
	// probability that CheY will spontaneously dephosphorylate
	ProbCheYAutoDephosPRI,
	ProbCheBAutoDephosPRI,

	// probability of motor turning CW when there are no ligand inputs
	ProbMotorCwPRI,

	// probability that MCP will bind to ligand during piston binding rule
	ProbPistonBindToLigandPRI, // NOT out of 10000

	// probability that CheB will demeth MCP even if piston is inactive
	ProbCheBdemethInactivePRI,
	// probability that CheR will meth MCP even if piston is active
	ProbCheRmethActivePRI,

	SizeOfProbIdEnumPRI // number of items in this enumeration
} ProbIdEnum;

const int ProbabilityMax = 10000; // maximum probability

// Probabilities
class Probabilities  
{
public:
	Probabilities();
	virtual ~Probabilities();
	int pr[SizeOfProbIdEnumPRI]; // array of probability values
};

#endif // !defined(AFX_PROBABILITIES_H__EFF44007_D82A_4FAF_80E3_09F56844BED9__INCLUDED_)
