/**************************************************************************
EASY MSc Dissertation - Project
Ken Webb
August 11, 2004
SmallMolecule.cpp: implementation of the SmallMolecule class.

**************************************************************************/

#include "SmallMolecule.h"
#include "Monomer.h"
#include "Rndm.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Small molecule alphabet
// the default alphabet from which to construct small molecules
int SmallMolecule::alphabet[] = {
SmA_MN, SmC_MN, SmD_MN, SmE_MN, SmF_MN, SmG_MN, SmH_MN, SmI_MN, SmK_MN, SmL_MN,
SmM_MN, SmN_MN, SmP_MN, SmQ_MN, SmR_MN, SmS_MN, SmT_MN, SmV_MN, SmW_MN, SmY_MN,
NullMN
};
int SmallMolecule::lenAlphabet = 20;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SmallMolecule::SmallMolecule() {} // don't use

// Construct an instance of a small molecule from its string
SmallMolecule::SmallMolecule( Grid *g, int xx, int yy, int r, int f, int gl,
							 Polymer* c, bool a, char* tStr, char* nm )
{
	grid = g;
	gridSizeX = grid->getNumCols();
	gridSizeY = grid->getNumRows();
	smString = NULL;
	lenString = 0;
	setX(xx);
	setY(yy);
	setRot(r);
	setFlip(f);
	setGridLayer(gl);
	setContainer(c);
	setAttached(a);
	setString(tStr);
	setName( nm );
	setStatsType( nm );
}

// Construct an instance of a "named" (numbered) small molecule type
SmallMolecule::SmallMolecule( Grid *g, int xx, int yy, int r, int f, int gl,
							 Polymer* c, bool a, enum namedSmEnum smType, char* nm )
{
	grid = g;
	gridSizeX = grid->getNumCols();
	gridSizeY = grid->getNumRows();
	smString = NULL;
	lenString = 0;
	setX(xx);
	setY(yy);
	setRot(r);
	setFlip(f);
	setGridLayer(gl);
	setContainer(c);
	setAttached(a);
	setString( smType );
	setName( nm );
	setStatsType( nm );
}

SmallMolecule::~SmallMolecule()
{

}

// The "named" small molecule strings.
int adenosineStr[] = { AdenosineMN, WaterMN, WaterMN, WaterMN };
int ampStr[] = { AdenosineMN, PhosphorylGroupMN, WaterMN, WaterMN };
int adpStr[] = { AdenosineMN, PhosphorylGroupMN, PhosphorylGroupMN, WaterMN };
int atpStr[] = { AdenosineMN, PhosphorylGroupMN, PhosphorylGroupMN, PhosphorylGroupMN };
int methStr[] = {MethBaseMN, MethylGroupMN };
int meth0Str[] = {MethBaseMN, WaterMN };

///////////////////////////////////////////////////
// Set string as one of the named small molecules.
//    AdenosineSM and ATP family
//    MethSM
///////////////////////////////////////////////////
void SmallMolecule::setString( enum namedSmEnum smType )
{
	switch (smType) {
	case AdenosineSM:
		setString( adenosineStr, 4 );
		break;
	case AmpSM:
		setString( ampStr, 4 );
		break;
	case AdpSM:
		setString( adpStr, 4 );
		break;
	case AtpSM:
		setString( atpStr, 4 );
		break;
	case MethSM: // Methanol
		setString( methStr, 2 );
		break;
	case Meth0SM:
		setString( meth0Str, 2 );
		break;
	default:
		break;
	}
}

//////////////////////////////////////////
// createRandomString
// Create a random small molecule string.
//////////////////////////////////////////
void SmallMolecule::createRandomString( int len )
{
	int* templateString = new int[len+1];
	memsetInt( templateString, WildCardMN, len );
	setString( templateString, len );
	delete templateString;
}

///////////////////////////////////////////////////////
// setString
// Set the small molecule string as an integer string.
///////////////////////////////////////////////////////
void SmallMolecule::setString( int* templateStr, int len )
{
	int rNum;
	int i;

	// create a new string for this small molecule
	if (smString != NULL)
		delete smString;
	smString = new int[len+1];
	lenString = len;

	// convert wild cards in template into final monomers
	for (i = 0; i < len; i++) {
		switch (templateStr[i]) {
		case WildCardMN:
			rNum = Rndm::getRandomInt( lenAlphabet-1 );
			smString[i] = alphabet[rNum];
			break;
		default:
			smString[i] = templateStr[i];
			break;
		}
	}
	smString[i] = NullMN;
	// save the string as an actual shape
	setShape( smString, 1, lenString );
	// setHydrophobic(); // might be hydrophobic or not, so can move through membrane

	// Initialize bindings array
	if (bindings != NULL)
		delete bindings;
	bindings = new int[len+1];
	memsetInt(bindings, NullBNG, len+1);

	// Initialize rules array; currently no rules for small molecules
	if (rules != NULL)
		delete rules;
	rules = new enum ruleEnum[len + 1];
	memsetInt((int*)rules, NullRUL, len+1);
}

//////////////////////////////////////////////////////////////////////////
// setString
// Allow the small molecule string to be specified as a character string.
//////////////////////////////////////////////////////////////////////////
void SmallMolecule::setString( char* charStr )
{
    int i;
	int len = strlen( charStr );
	int* intStr = new int[len+1];
	for (i = 0; i < len; i++) {
		switch (charStr[i]) {
		case 'A': intStr[i] = SmA_MN; break;
		case 'C': intStr[i] = SmC_MN; break;
		case 'D': intStr[i] = SmD_MN; break;
		case 'E': intStr[i] = SmE_MN; break;
		case 'F': intStr[i] = SmF_MN; break;
		case 'G': intStr[i] = SmG_MN; break;
		case 'H': intStr[i] = SmH_MN; break;
		case 'I': intStr[i] = SmI_MN; break;
		case 'K': intStr[i] = SmK_MN; break;
		case 'L': intStr[i] = SmL_MN; break;
		case 'M': intStr[i] = SmM_MN; break;
		case 'N': intStr[i] = SmN_MN; break;
		case 'P': intStr[i] = SmP_MN; break;
		case 'Q': intStr[i] = SmQ_MN; break;
		case 'R': intStr[i] = SmR_MN; break;
		case 'S': intStr[i] = SmS_MN; break;
		case 'T': intStr[i] = SmT_MN; break;
		case 'V': intStr[i] = SmV_MN; break;
		case 'W': intStr[i] = SmW_MN; break;
		case 'Y': intStr[i] = SmY_MN; break;
		case '#': intStr[i] = WildCardMN; break;
		default: intStr[i] = NullMN; break;
		}
	}
	intStr[i] = NullMN;
	setString( intStr, len );
	delete intStr;
}

///////////////////////////
// getString
// Get the integer string.
///////////////////////////
int* SmallMolecule::getString( void )
{
	return (smString);
}

//////////////////////////////////////////
// act
// do a possible binding or other action.
//////////////////////////////////////////
void SmallMolecule::act( void )
{
	//printf( "Protein::act()\n" );

}

//////////////////////////////////
// move
// Do a possible random movement.
//////////////////////////////////
void SmallMolecule::move( void )
{
	if (attached)
		return;
	int rNum = Rndm::getRandomInt( 1, SizeOfActionEnumACT-1 );
	switch (rNum) {
	case MoveRightACT:
		moveRight();
		break;
	case MoveLeftACT:
		moveLeft();
		break;
	case MoveUpACT:
		moveUp();
		break;
	case MoveDownACT:
		moveDown();
		break;
	case RotateCwACT:
		rotateCw();
		break;
	case RotateCcwACT:
		rotateCcw();
		break;
	case FlipVertACT:
		flipVert();
		break;
	case FlipHorzACT:
		flipHorz();
		break;
	default: // 0
		// safe to ignore this; protein does nothing during this time step
		// printf( "ERROR: SM::act() default %d\n", rNum );
		break;
	}
}

////////////////////////////////////////////////////////
//                                       Movements    //
////////////////////////////////////////////////////////

/////////////
// moveRight
/////////////
void SmallMolecule::moveRight( void )
{
	if (x+1 < gridSizeX) {
		x++;
		if (doesOverlapMembrane(grid)) {
			x--;
		}
	}
}


/////////////
// moveLeft
/////////////
void SmallMolecule::moveLeft( void )
{
	if (x-1 >= 0) {
		x--;
		if (doesOverlapMembrane(grid)) {
			x++;
		}
	}
}

//////////
// moveUp
//////////
void SmallMolecule::moveUp( void )
{
	if (y+1 < gridSizeY) {
		y++;
		if (doesOverlapMembrane(grid)) {
			y--;
		}
	}
}

/////////////
// moveDown
/////////////
void SmallMolecule::moveDown( void )
{
	if (y-1 >= 0) {
		y--;
		if (doesOverlapMembrane(grid)) {
			y++;
		}
	}
}

/////////////////////
// rotateCw
// Rotate clockwise.
/////////////////////
void SmallMolecule::rotateCw( void )
{
	incRot();
	if (doesOverlapMembrane(grid)) {
		decRot();
	}
}

/////////////////////////////
// rotateCcw
// Rotate counter-clockwise.
/////////////////////////////
void SmallMolecule::rotateCcw( void )
{
	decRot();
	if (doesOverlapMembrane(grid)) {
		incRot();
	}
}

////////////////////
// flipVert
// Flip vertically.
////////////////////
void SmallMolecule::flipVert( void )
{
	incFlip();
}

//////////////////////
// flipVert
// Flip horizontally.
//////////////////////
void SmallMolecule::flipHorz( void )
{
	incFlip();
}

//////////////////////////////////////////
// inventory
// Write inventory information to a file.
//////////////////////////////////////////
void SmallMolecule::inventory( FILE *fp, Monomers *mn )
{
	int row, col;
	bool evenRow = true; // even row is row 0, 2, 4, etc.
	int mon; // monomer
	int bin; // binding
	int rul; // rule
	char brOut[100]; // bindings and rules info
	memset(brOut, 0, 100);
	int count = 0;
	if (getGhost())
		fprintf( fp, "Sg,%5d,", id );
	else
		fprintf( fp, "S,%5d,", id );
	// output the shape
	for (row = 0; row < numRows; row++) {
		for (col = 0; col < numCols; col++) {
			if (evenRow) {
				mon = getShapeVal( row, col );
				bin = getBindingVal( row, col );
				rul = getRuleVal( row, col );
			}
			else {
				mon = getShapeVal( row, numCols-col-1 );
				bin = getBindingVal( row, numCols-col-1 );
				rul = getRuleVal( row, numCols-col-1 );
			}
			if ((mon != NullMN) && (mon != WaterMN)) {
				fprintf( fp, "%c", mn->get(mon)->code );
				if (bin == NullBNG) {
					if (rul == NullRUL)
						brOut[count] = ' '; // no binding, no rule
					else {
						if (rul == NoMoreRulesRUL)
							brOut[count] = '.';
						else
							brOut[count] = '='; // no binding, has rule
					}
				}
				else { // has a binding
					if (rul == NullRUL)
						brOut[count] = '|'; // has binding, no rule
					else {
						if (rul == NoMoreRulesRUL)
							brOut[count] = '|';
						else
							brOut[count] = '#'; // has binding, has rule
					}
				}
				count++;
			}
		}
		if (evenRow) evenRow = false;
		else evenRow = true;
	}
	fprintf( fp, ",%s\n", getName() );
	fprintf( fp, " ,     ,%s, \n", brOut );
}

/////////////////////////////////////////////////////
// doStats
// Save some statistical information each time step.
/////////////////////////////////////////////////////
void SmallMolecule::doStats( void )
{
	int i;
	switch( statsType ) {
	case Ligands_ST: // number of ligands
		grid->stats->cnt[Ligands_ST]++;
		break;
	case PhosFree_ST: // number of phosphoryl groups
		for (i = 1; i < numCols; i++)
			if (getShapeVal(0,i) == PhosphorylGroupMN)
				grid->stats->cnt[PhosFree_ST]++;
		break;
	case MethFree_ST: // mumber of methyl groups
		if (getShapeVal(0,1) == MethylGroupMN)
			grid->stats->cnt[MethFree_ST]++;
		break;
	default:
		break;
	}
}
