/**************************************************************************
EASY MSc Dissertation - Project
Ken Webb
August 4, 2004
SmallMolecule.h: interface for the SmallMolecule class.

A SmallMolecule is a subclass of Polymer.
Small molecules include ligands, ATP, and methanol.
**************************************************************************/

#if !defined(AFX_SMALLMOLECULE_H__BD5EFACF_4C6A_43E3_AD57_5F49BB22B8F0__INCLUDED_)
#define AFX_SMALLMOLECULE_H__BD5EFACF_4C6A_43E3_AD57_5F49BB22B8F0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Polymer.h"
#include "Grid.h"

// Named small molecules
typedef enum namedSmEnum {
	AdenosineSM = 1,
	AmpSM, // AMP
	AdpSM, // ADP
	AtpSM, // ATP
	MethSM, // free Methanol
	Meth0SM, // a base for attachment of Methanol
	SizeOfNamedSmEnum // Size of enum
} namedSmEnum;

// SmallMolecule
class SmallMolecule : public Polymer  
{
public:
	SmallMolecule();
	SmallMolecule( Grid *g, int xx, int yy, int r, int f, int gl,
		Polymer* c, bool a, char* tStr, char* nm = "Sm" );
	SmallMolecule( Grid *g, int xx, int yy, int r, int f, int gl,
		Polymer* c, bool a, enum namedSmEnum smType, char* nm = "Sm" );
	virtual ~SmallMolecule();
	// A small molecule is an unfolded string of monomers
	void createRandomString( int len );
	void setString( int* templateStr, int len );
	void setString( char* charStr );
	int* getString( void );
	// A small molecule moves every time step, and could take additional actions but doesn't
	void act( void );
	void move( void );
	// Inventory
	void inventory( FILE *fp, Monomers *mn );

protected:
	Grid *grid; // ptr to global grid
	int *smString; // small molecule string
	int lenString; // length of string
	// Alphabet of valid constituent monomers
	static int alphabet[];
	static int lenAlphabet;

	// Actions
	void moveRight( void );
	void moveLeft( void );
	void moveUp( void );
	void moveDown( void );
	void rotateCw( void );
	void rotateCcw( void );
	void flipVert( void );
	void flipHorz( void );

	void setString( enum namedSmEnum smType );
	// Stats
	void doStats( void );
};

#endif // !defined(AFX_SMALLMOLECULE_H__BD5EFACF_4C6A_43E3_AD57_5F49BB22B8F0__INCLUDED_)
