/**************************************************************************
EASY MSc Dissertation - Project
Ken Webb
August 4, 2004
Statistics.h: interface for the Statistics class.

Statistics are optionally gathered on the system during each time step,
and saved to various files for later analysis.
One instance of Statistics for each instance of BacteriumApp.
All stats member variables are public.
**************************************************************************/

#if !defined(AFX_STATISTICS_H__52FB3BB5_2B24_4E90_A9CC_080B465B6638__INCLUDED_)
#define AFX_STATISTICS_H__52FB3BB5_2B24_4E90_A9CC_080B465B6638__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Probabilities.h"
#include <stdio.h>

const char statsSeries = 'b'; // first letter of file names

// Statistics Types
// This enumeration contains all types of statistics that could be gathered.
// Those that are not currently useful are commented out.
typedef enum StatsTypeEnum {
	Null_ST = 0,
	// MCP
	Mcp_ST, // number of MCP proteins
	Mcp_PistonBound_ST, // number pistons bound to a ligand
	Mcp_Methylated_ST, // number methylations (each MCP has 0 to 8)
	//Mcp_CheWBound_ST,  // number bound to CheW (should always be 4)
	//Mcp_CheABound_ST, // number bound to CheA (should always be 4)

	// CheW
	CheW_ST,
	//CheW_McpBound_ST,

	// CheA
	CheA_ST,
	CheA_Phosphorylated_ST, // number phosphorylated (each CheA is 0 or 1)
	//CheA_McpBound_ST,
	CheA_CheYbBound_ST, // number bound to CheY or CheB

	// CheY
	CheY_ST,
	CheY_Phosphorylated_ST, // number phosphorylated (each CheY is 0 or 1)
	//CheY_SwappedMon_ST, // number with swapped monomers (NL -> LN)
	//CheY_SwappedRule_ST, // number with SwapIfUnphosphorylatedRUL
	CheY_CheABound_ST, // number bound to CheA
	CheY_MotorBound_ST, // number bound to motors

	// Motor Complex
	Motor_ST,
	//Motor_Cw_ST, // number rotating clockwise
	Motor_CheYBound_ST, // number bound to CheY (each Motor is 0 to 3)

	// CheB
	CheB_ST,
	CheB_Phosphorylated_ST, // number phosphorylated (each CheB is 0 or 1)
	//CheB_SwappedMon_ST, // number with swapped monomers (ED -> DE)
	//CheB_SwappedRule_ST, // number with SwapIfUnphosphorylatedRUL
	CheB_CheABound_ST, // number bound to CheA
	//CheB_McpDemethylated_ST, // number MCP demethylated this time step

	// CheR
	CheR_ST,
	//CheR_McpMethylated_ST, // number MCP methylated this time step

	// CheZ
	CheZ_ST,
	//CheZ_CheYDephosphorylated_ST, // number CheY dephosphorylated this time step

	// Small molecules
	Ligands_ST, // number of unghosted ligands
	PhosFree_ST, // Free phosphoryl: Adenosine_ST, Amp_ST, Adp_ST, Atp_ST,
	MethFree_ST, // Free Methanol: Meth0_ST, Meth_ST
	SizeOfStatsTypeEnum_ST
} StatsTypeEnum;

// Statistics
class Statistics  
{
public:
	Statistics( char* pn, char* fnp, Probabilities *pr );
	virtual ~Statistics();

	// counts of various things during a single time step
	int cnt[SizeOfStatsTypeEnum_ST];

	void init( void ); // inititialize array to all zeros
	void output( void ); // output to file
	void outputHeadings( void ); // output headings to file
	void outputGnuplotScriptS( void ); // output a gnuplot stats script
	void outputGnuplotScriptR( char* fnp ); // output a gnuplot runs script
	void createNotesFile( char* fnp ); // create a .txt file containing all parameters
	void outputEnvAppHist( int bactIx, int* ligCount, int nLig ); // output EnvironmentApp histogram
	char* getPathName( void ); // get the path string
	char* getDateTime( void ); // get date and time as a string
	void setMask( char* m ); // set/get mask for which stats will be saved to file
	char* getMask( void );
	bool statsEnabled; // whether or not stats are enabled for the entire application

protected:
	Probabilities *prob; // probabilities to save out in stats files
	FILE *fp;
	FILE *fpPlt;
	char* mask; // mask of which stats to save out
	static char* head[];
	char dateTime[20]; // dateTime string to use in file names
	char* pathName;
	char dataFileName[100];
	char* fileNamePrefix;
	bool firstTime;
	void setDateTime( void ); // create date and time string
};

#endif // !defined(AFX_STATISTICS_H__52FB3BB5_2B24_4E90_A9CC_080B465B6638__INCLUDED_)
