/**************************************************************************
EASY MSc Dissertation - Project
Ken Webb
pbnnMain.cpp
August 7, 2004

Protein-Based Neural Networks (PBNN).
Mainline in PBNN system.
Creates one or more apps (client BacteriumApp and/or server EnvironmentApp),
each with its own OpenGL window.
**************************************************************************/

#include "BacteriumApp.h"
#include "EnvironmentApp.h"
#include "Probabilities.h"
#include "Statistics.h"
#include "Msg.h"
#include "Rndm.h"
#include "GlutMaster.h"
#include "Queue.h"
#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h> // only needed for quick tests

// Queue sizes
const int QueueSize = 1000;

// C Function Prototypes
void createProb( int probID, Probabilities* prob );
void testRandom( void );

// main function
int main( int argc, char *argv[] )
{
	if (argc < 3) {
		printf("usage: Pbnn <numBactAppInstances> <numEnvAppInstances>\n" );
		printf("  normally: Pbnn 1 0\n" );
		exit(0);
	}

	Rndm::seedRandomNumberGenerator();
	//testRandom(); // uncomment to test random functions

	// create object that allows for multiple OpenGL windows
	GlutMaster* glutMaster;
	glutMaster = new GlutMaster();    

	int numBactAppInstances = atoi(argv[1]);
	int numEnvAppInstances  = atoi(argv[2]);

	// create Probabilities instances; be able to pass separate probs to each app
	Probabilities *prob1, *prob2;
	prob1 = new Probabilities();
	createProb( 1, prob1 );

	// create Statistics instances
	Statistics *stats1, *stats2, *statsE;
	stats1 = new Statistics( "..\\..\\Stats\\", "stats1", prob1 );

	// create Queues to allow each client to communicate with server
	Queue *bactEnvQ1, *bactEnvQ2, *envBactQ1, *envBactQ2;
	bactEnvQ1 = new Queue( QueueSize ); // Q able to hold up to n items
	envBactQ1 = new Queue( QueueSize );

	// create a new Bacterium application (client)
	BacteriumApp *client1;
	client1 = new BacteriumApp( glutMaster, prob1, stats1 );
	client1->setQ( bactEnvQ1, Q_OUT );
	client1->setQ( envBactQ1, Q_IN );
	// read in parameters from file
	client1->initialize( "..\\ConfigFileBA_1.txt" );
	Msg::debugM = false;
	client1->glMain(argc, argv);

	// optionally create another new Bacterium application (client)
	if (numBactAppInstances > 1) {
		bactEnvQ2 = new Queue( QueueSize );
		envBactQ2 = new Queue( QueueSize );
		prob2 = new Probabilities();
		stats2 = new Statistics( "..\\..\\Stats\\", "stats2", prob2 );
		createProb( 2, prob2 );
		BacteriumApp *client2;
		client2 = new BacteriumApp( glutMaster, prob2, stats2 );
		client2->setQ( bactEnvQ2, Q_OUT );
		client2->setQ( envBactQ2, Q_IN );
		client2->initialize( "..\\ConfigFileBA_2.txt" );
		client2->glMain(argc, argv);
	}

	// optionally create a new Environment application (server)
	// should never be more than one of these
	EnvironmentApp *server;
	if (numEnvAppInstances == 1) {
		// EnvironmentApp doesn't need probabilities for now
		statsE = new Statistics( "..\\..\\Stats\\", "statsE", prob1 );
		server = new EnvironmentApp( glutMaster, statsE );
		server->setQ( bactEnvQ1, envBactQ1 );
		if (numBactAppInstances > 1)
			server->setQ( bactEnvQ2, envBactQ2 );
		// read in parameters from file
		server->initialize( "..\\ConfigFileE.txt" );
		server->glMain(argc, argv);
	}

	// start the OpenGL processing loop
	glutMaster->SetTimerFunc(1000, 50, 17);
	glutMaster->CallGlutMainLoop();

	delete server;
	return 0;
}

// Create probabilities
// should be able to use a GA to do this
// these are just defaults that will (almost certainly) be replaced from a file
void createProb( int probID, Probabilities* prob )
{
	switch (probID) {
	case 1:
		prob->pr[ProbLigandUnbindPRI]   =  50; // 50
		prob->pr[ProbCheAAutoPhosPRI]   = 255; // 255
		prob->pr[ProbCheYAutoDephosPRI] =  10; // 10
		prob->pr[ProbCheBAutoDephosPRI] = 10; // 100
		prob->pr[ProbMotorCwPRI]        = 800; // 800
		prob->pr[ProbPistonBindToLigandPRI] = 255; // 255
		prob->pr[ProbCheBdemethInactivePRI] = 100;
		prob->pr[ProbCheRmethActivePRI] = 100;
		break;
	case 2:
		prob->pr[ProbLigandUnbindPRI]   =  50; // 50
		prob->pr[ProbCheAAutoPhosPRI]   = 255; // 255
		prob->pr[ProbCheYAutoDephosPRI] =  10; // 10
		prob->pr[ProbCheBAutoDephosPRI] = 100; // 100
		prob->pr[ProbMotorCwPRI]        = 800; // 800
		prob->pr[ProbPistonBindToLigandPRI] = 255; // 255
		prob->pr[ProbCheBdemethInactivePRI] = 100;
		prob->pr[ProbCheRmethActivePRI] = 100;
		break;
	default:
		printf( "ERROR: createProb() default = %d", probID );
		break;
	}
}

// various tests of the random number functions
// to use, uncomment
void testRandom( void )
{
	// Test random number generator
	// To use this feature:
	//    App4 1 0 > ..\..\Stats\rand.txt
	//    start gnuplot
	//      plot 'rand.txt'
	//Rndm::testRandom(99, 123, 100000); // inclusive
	//return 0;

	// Test left shift << for methLevel
	//for (int ml = 1; ml <= 8; ml++) {
	//	printf( "%d %d\n", ml, ((2 << (ml-1)) - 1));
	//}
	//return 0;

	// Test sigmoidal function for methLevel
	//for (int ml = -4; ml <= 4; ml++) {
	//	printf( "%d %d\n", ml, ((int)((1.0f / (1.0f + exp((double)-ml))) * 128.0f) - 4) );
	//}
	//return 0;

}
