/**************************************************************************
EASY MSc Dissertation - Project
Ken Webb
August 4, 2004
Protein.h: interface for the Protein class.

A Protein is a subclass of Polymer.
Proteins are the active objects in the PBNN system, in that they have rules.
Proteins are composed of amino acid monomers.
**************************************************************************/

#if !defined(AFX_PROTEIN_H__E264C5A7_BBB8_46BE_BD0F_C6A0510B74E0__INCLUDED_)
#define AFX_PROTEIN_H__E264C5A7_BBB8_46BE_BD0F_C6A0510B74E0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Polymer.h"
#include "Grid.h"
#include "Monomer.h"
#include "Probabilities.h"

class Protein : public Polymer
{
public:
	Protein();
	Protein( Grid *g, int xx, int yy, int r, int f, int gl,
		Polymer* c, bool a, char* tStr, Probabilities* pr, char* nm = "Prot" );
	virtual ~Protein();

	// String initialization; each protein is originally just a string of amino acids
	void createRandomString( int len );
	void setString( int* templateStr, int len );
	void setString( char* templateStr );
	int* getString( void );

	// Piston
	void setPistonOffset( int rNum, int val );
	int getPistonOffset( int rNum );

	void printString( void );

	// Actions performed each time step
	void act( void );
	void move( void );
	void doRule( enum ruleEnum, int row, int col );
	// Phosphorylation
	bool isPhosphorylated( int row, int col );
	void phosphorylate( int row, int col );
	void dephosphorylate( int row, int col );
	// Methylation
	bool isMethylated( int row, int col );
	void methylate( int row, int col );
	void demethylate( int row, int col );
	int getSummedMethLevel( void );
	int getMethLevel( int mRow, int mCol );
	// Inventory
	void inventory( FILE *fp, Monomers *mn );
	// Probabilities
	void setProbabilities( Probabilities* pr );
	Probabilities* getProbabilities( void );
	void setProbVal( int id, int val );
	int getProbVal( int id );
protected:
	Grid *grid; // ptr to global grid object
	Probabilities* prob; // all proteins within a BacteriumApp have same prob
	int* pistonOffset; // offsets when shape substrings act as pistons
	// Hydrophobic
	int hydrophobicCol; // first column in hydrophobic region, if is hydrophobic
	bool isColHydrophobic(int col);
	bool isHydrophobic( int c);
	void setHydrophobic( void );

	int *proteinString; // unfolded string
	int lenString; // length of unfolded string
	int numG; // number of G symbols in the protein string

	// Alphabet of valid constituent monomers
	static int alphabet[];
	static int lenAlphabet;

	// Four types of amino acid
	static int alphHydrophobic[]; // hydrophobic amino acids
	static int alphCharged[]; // charged amino acids
	static int alphPolar[]; // polar amino acids
	static int alphGlycine[]; // glycine amino acids
	static int lenAlphHydrophobic;
	static int lenAlphCharged;
	static int lenAlphPolar;

	// Actions
	void moveRight( void );
	void moveLeft( void );
	void moveUp( void );
	void moveDown( void );
	void rotateCw( void );
	void rotateCcw( void );
	void flipVert( void );
	void flipHorz( void );

	void foldString( void ); // fold a string of amino acids into a 2D shape
	// Calculate new position of protein such that its hydrophobic domain straddles the membrane
	int getHydroDiff( int moveNum, Polymer* pol );

	// Match
	bool matchP_P( MonomerTypeEnum monP1, MonomerTypeEnum monP2 );
	bool matchP_SM( MonomerTypeEnum monP, MonomerTypeEnum monSM );

	// Stats
	void doStats( void );

	// Rules
	void doRule_PistonBinding( enum ruleEnum ruleNum, int rRow, int rCol );
	void doRule_ProteinProteinBinding( enum ruleEnum ruleNum, int rRow, int rCol );
	void doRule_ProteinMembraneBinding( enum ruleEnum ruleNum, int rRow, int rCol );
	void doRule_PhosphorylationCheA3( enum ruleEnum ruleNum, int rRow, int rCol );
	void doRule_SwapOnPhosChange( enum ruleEnum ruleNum, int rRow, int rCol );
	void doRule_AutoPhosphorylation( enum ruleEnum ruleNum, int rRow, int col );
	void doRule_AutoDephosphorylation( enum ruleEnum ruleNum, int rRow, int col );
	void doRule_Motor( enum ruleEnum ruleNum, int rRow, int rCol );
	void doRule_MethylTransferase( enum ruleEnum ruleNum, int rRow, int rCol );
	void doRule_MethylEsterase( enum ruleEnum ruleNum, int rRow, int rCol );
	void doRule_Phosphatase( enum ruleEnum ruleNum, int rRow, int rCol );
};

#endif // !defined(AFX_PROTEIN_H__E264C5A7_BBB8_46BE_BD0F_C6A0510B74E0__INCLUDED_)
